// GT_HelloWorldWin32.cpp
// compile with: /D_UNICODE /DUNICODE /DWIN32 /D_WINDOWS /c

#include <windows.h>
#include <stdlib.h>
#include <string.h>
#include <tchar.h>

#include <iostream>
#include <string>

#include "../resource.h"
#include "ui_mode.h"
#include "video_player.h"


CUiMode *ui_mode_ = NULL;
CVideoPlayer *local_video_render_ = NULL;
CVideoPlayer *remote_video_render_ = NULL;


static const std::string kPushParam = "{\
    \"local_address\":\"192.168.42.146\",\n\
    \"local_port_video\":9001,\n\
    \"local_port_audio\":9002,\n\
    \"remote_address\":\"192.168.42.121\",\n\
    \"remote_port_video\":10001,\n\
    \"remote_port_audio\":10002\n\
}";

static const std::string kPullParam = "{\
    \"local_address\":\"192.168.42.121\",\n\
    \"local_port_video\":10001,\n\
    \"local_port_audio\":10002,\n\
    \"remote_address\":\"192.168.42.146\",\n\
    \"remote_port_video\":9001,\n\
    \"remote_port_audio\":9002\n\
}";

int pull_handle_ = 0;
int push_handle_ = 0;

HWND	main_hwnd_ = 0;
HWND	local_video_hwnd_ = 0;
HWND	remote_video_hwnd_ = 0;

INT_PTR WINAPI Main_Dialog(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_INITDIALOG:
	{
		main_hwnd_ = hWnd;

		local_video_hwnd_ = ::GetDlgItem(hWnd, IDC_STATIC_VIDEO);
		remote_video_hwnd_ = ::GetDlgItem(hWnd, IDC_STATIC_VIDEO_REMOTE);

		ui_mode_ = new CUiMode((int)hWnd);
		ui_mode_->Initialize();

		local_video_render_ = new CVideoPlayer((int)local_video_hwnd_, 1920, 1080);
		remote_video_render_ = new CVideoPlayer((int)remote_video_hwnd_, 1920, 1080);
	}
	break;

	case WM_USER_DRAW_REMOTE_VIDEO:
	{
		int width = 0;
		int height = 0;

		ui_mode_->GetMeRemoteVideoFrame(remote_video_render_->video_frame_buffer_y_, remote_video_render_->frame_y_pitch_,
			remote_video_render_->video_frame_buffer_u_, remote_video_render_->frame_u_pitch_,
			remote_video_render_->video_frame_buffer_v_, remote_video_render_->frame_v_pitch_,
			width, height);

		remote_video_render_->PlayVideo(width, height);
	}
	break;

	case WM_USER_DRAW_LOCAL_VIDEO:
	{
		int width = 0;
		int height = 0;

		ui_mode_->GetMeLocalVideoFrame(local_video_render_->video_frame_buffer_y_, local_video_render_->frame_y_pitch_,
			local_video_render_->video_frame_buffer_u_, local_video_render_->frame_u_pitch_,
			local_video_render_->video_frame_buffer_v_, local_video_render_->frame_v_pitch_,
			width, height);

		local_video_render_->PlayVideo(width, height);
	}
	break;

	case WM_COMMAND:
	{
		int wmId = LOWORD(wParam);

		switch (wmId)
		{
		case IDC_BUTTON_PUSH:
		{
			push_handle_ = ui_mode_->CreateRoleHandle(kPushParam.c_str(), ZRoleType::Z_ROLE_TYPE_PUBLISH);
			ui_mode_->StartPush(push_handle_);
		}
		break;

		case IDC_BUTTON_PULL:
		{
			pull_handle_ = ui_mode_->CreateRoleHandle(kPullParam.c_str(), ZRoleType::Z_ROLE_TYPE_SUBSCRIBE);
			ui_mode_->StartPull(pull_handle_);
		}
		break;

		}
	}
	break;

	}
	return 0;
}

int WINAPI WinMain(HINSTANCE hInstance,
                   HINSTANCE hPrevInstance,
                   LPSTR lpCmdLine,
                   int nCmdShow)
{
	
	DialogBoxParam(hInstance, MAKEINTRESOURCE(IDD_DIALOG_MAIN), NULL, Main_Dialog, _ttoi(""));

	return 0;
}