#include "ui_mode.h"
#include <Windows.h>

static const int kWidth = 1080;
static const int kHeight = 720;

CUiMode::CUiMode(int hWnd)
{
	win_hwnd_ = hWnd;

	push_handle_ = 0;
	pull_handle_ = 0;

	//local
	width_ = 0;
	height_ = 0;

	local_buffer_y_ = new unsigned char[kWidth * kHeight];
	local_buffer_u_ = new unsigned char[kWidth * kHeight];
	local_buffer_v_ = new unsigned char[kWidth * kHeight];

	remote_width_ = 0;
	remote_height_ = 0;

	remote_buffer_y_ = new unsigned char[kWidth * kHeight];
	remote_buffer_u_ = new unsigned char[kWidth * kHeight];
	remote_buffer_v_ = new unsigned char[kWidth * kHeight];
}
CUiMode::~CUiMode()
{
	//local
	delete local_buffer_y_;
	delete local_buffer_u_;
	delete local_buffer_v_;

	local_buffer_y_ = nullptr;
	local_buffer_u_ = nullptr;
	local_buffer_v_ = nullptr;

	//remote
	delete remote_buffer_y_;
	delete remote_buffer_u_;
	delete remote_buffer_v_;

	remote_buffer_y_ = nullptr;
	remote_buffer_u_ = nullptr;
	remote_buffer_v_ = nullptr;
}
void CUiMode::Initialize()
{
	offcn::Initialize(this);
}
int CUiMode::CreateRoleHandle(const char *param, ZRoleType type)
{
	int handle = offcn::CreateRoleHandle(param, this, type);

	if (type == Z_ROLE_TYPE_PUBLISH)
	{
		push_handle_ = handle;
	}
	else
	{
		pull_handle_ = handle;
	}
	return handle;
}
void CUiMode::DestroyRoleHandle(int handle, ZRoleType type)
{
	offcn::DestroyRoleHandle(handle, type);
}
void CUiMode::StartPush(int handle)
{
	offcn::StartPublish(handle);
}
void CUiMode::StopPush(int handle)
{
	offcn::StopPublish(handle);
}
void CUiMode::StartPull(int handle)
{
	offcn::StartSubscribe(handle);
}
void CUiMode::StopPull(int handle)
{
	offcn::StopSubscribe(handle);
}

void CUiMode::GetMeLocalVideoFrame(unsigned char *dst_buffer_y, int &y_pitch, unsigned char *dst_buffer_u, int &u_pitch, unsigned char *dst_buffer_v, int &v_pitch, int &width, int &height)
{
	readLocalBuffer(dst_buffer_y, y_pitch, dst_buffer_u, u_pitch, dst_buffer_v, v_pitch, width, height);
}
void CUiMode::writeLocalBuffer(const unsigned char *pY, int y_pitch, const unsigned char *pU, int u_pitch, const unsigned char *pV, int v_pitch, int width, int height)
{
	std::unique_lock<std::mutex> lck(local_mutex_);

	local_y_pitch_ = y_pitch;
	memcpy(local_buffer_y_, pY, y_pitch*height);

	local_u_pitch_ = u_pitch;
	memcpy(local_buffer_u_, pU, u_pitch*height / 2);

	local_v_pitch_ = v_pitch;
	memcpy(local_buffer_v_, pV, v_pitch*height / 2);

	width_ = width;
	height_ = height;
}
void CUiMode::readLocalBuffer(unsigned char *dst_buffer_y, int &y_pitch, unsigned char *dst_buffer_u, int &u_pitch, unsigned char *dst_buffer_v, int &v_pitch, int &width, int &height)
{
	std::unique_lock<std::mutex> lck(local_mutex_);

	width = width_;
	height = height_;

	y_pitch = local_y_pitch_;
	u_pitch = local_u_pitch_;
	v_pitch = local_v_pitch_;

	memcpy(dst_buffer_y, local_buffer_y_, y_pitch*height);
	memcpy(dst_buffer_u, local_buffer_u_, u_pitch*height / 2);
	memcpy(dst_buffer_v, local_buffer_v_, v_pitch*height / 2);
}

void CUiMode::GetMeRemoteVideoFrame(unsigned char *dst_buffer_y, int &y_pitch, unsigned char *dst_buffer_u, int &u_pitch, unsigned char *dst_buffer_v, int &v_pitch, int &width, int &height)
{
	readRemoteBuffer(dst_buffer_y, y_pitch, dst_buffer_u, u_pitch, dst_buffer_v, v_pitch, width, height);
}

void CUiMode::writeRemoteBuffer(const unsigned char *pY, int y_pitch, const unsigned char *pU, int u_pitch, const unsigned char *pV, int v_pitch, int width, int height)
{
	std::unique_lock<std::mutex> lck(remote_mutex_);

	remote_y_pitch_ = y_pitch;
	memcpy(remote_buffer_y_, pY, y_pitch*height);

	remote_u_pitch_ = u_pitch;
	memcpy(remote_buffer_u_, pU, u_pitch*height / 2);

	remote_v_pitch_ = v_pitch;
	memcpy(remote_buffer_v_, pV, v_pitch*height / 2);

	remote_width_ = width;
	remote_height_ = height;
}
void CUiMode::readRemoteBuffer(unsigned char *dst_buffer_y, int &y_pitch, unsigned char *dst_buffer_u, int &u_pitch, unsigned char *dst_buffer_v, int &v_pitch, int &width, int &height)
{
	std::unique_lock<std::mutex> lck(remote_mutex_);

	width = remote_width_;
	height = remote_height_;

	y_pitch = remote_y_pitch_;
	u_pitch = remote_u_pitch_;
	v_pitch = remote_v_pitch_;

	memcpy(dst_buffer_y, remote_buffer_y_, y_pitch*height);
	memcpy(dst_buffer_u, remote_buffer_u_, u_pitch*height / 2);
	memcpy(dst_buffer_v, remote_buffer_v_, v_pitch*height / 2);
}

void CUiMode::OnGlobalEvent(int code, const char *context)
{

}
void CUiMode::OnChannelEvent(int handle, int code, const char *context)
{

}
void CUiMode::OnChannelVideoFrame(int handle, const unsigned char *pY, int yPitch, const unsigned char *pU, int uPitch, const unsigned char *pV, int vPitch, int width, int height)
{
	if (handle == push_handle_)
	{
		writeLocalBuffer(pY, yPitch, pU, uPitch, pV, vPitch, width, height);
		::PostMessage((HWND)win_hwnd_, WM_USER_DRAW_LOCAL_VIDEO, 0, 0);
	}
	else
	{
		writeRemoteBuffer(pY, yPitch, pU, uPitch, pV, vPitch, width, height);
		::PostMessage((HWND)win_hwnd_, WM_USER_DRAW_REMOTE_VIDEO, 0, 0);
	}
}
void CUiMode::OnChannelAudioFrame(int handle, const void* audio_data, int bits_per_sample, int sample_rate, size_t number_of_channels, size_t number_of_frames, bool is_remote)
{

}
