#pragma once

#include "ztypes.h"
#include "libmediacore.h"

#include <mutex>

#define	WM_USER_DRAW_REMOTE_VIDEO	WM_USER + 100
#define	WM_USER_DRAW_LOCAL_VIDEO	WM_USER + 101

class CUiMode : public GlobalObserver, public ChannelObserver
{
public:
	CUiMode(int hWnd);
	~CUiMode();

	void Initialize();
	int CreateRoleHandle(const char *param, ZRoleType type);
	void DestroyRoleHandle(int handle, ZRoleType type);
	void StartPush(int handle);
	void StopPush(int handle);
	void StartPull(int handle);
	void StopPull(int handle);

	/**
	 *	GlobalObserver
	 */
	virtual void OnGlobalEvent(int code, const char *context);

	/**
	 *	ChannelObserver
	 */
	virtual void OnChannelEvent(int handle, int code, const char *context);
	virtual void OnChannelVideoFrame(int handle, const unsigned char *pY, int yPitch, const unsigned char *pU, int uPitch, const unsigned char *pV, int vPitch, int width, int height);
	virtual void OnChannelAudioFrame(int handle, const void* audio_data, int bits_per_sample, int sample_rate, size_t number_of_channels, size_t number_of_frames, bool is_remote);

	/**
	 *	UI thread get local capture video frame to play
	 */
	void GetMeLocalVideoFrame(unsigned char *dst_buffer_y, int &y_pitch, unsigned char *dst_buffer_u, int &u_pitch, unsigned char *dst_buffer_v, int &v_pitch, int &width, int &height);

	/**
	 *	UI thread get remote video frame to play
	 */
	void GetMeRemoteVideoFrame(unsigned char *dst_buffer_y, int &y_pitch, unsigned char *dst_buffer_u, int &u_pitch, unsigned char *dst_buffer_v, int &v_pitch, int &width, int &height);

private:
	void writeLocalBuffer(const unsigned char *pY, int y_pitch, const unsigned char *pU, int u_pitch, const unsigned char *pV, int v_pitch, int width, int height);
	void readLocalBuffer(unsigned char *dst_buffer_y, int &y_pitch, unsigned char *dst_buffer_u, int &u_pitch, unsigned char *dst_buffer_v, int &v_pitch, int &width, int &height);

	unsigned char *local_buffer_y_;
	unsigned char *local_buffer_u_;
	unsigned char *local_buffer_v_;
	int local_y_pitch_;
	int local_u_pitch_;
	int local_v_pitch_;

	int width_;
	int height_;

	std::mutex	local_mutex_;

private:
	void writeRemoteBuffer(const unsigned char *pY, int y_pitch, const unsigned char *pU, int u_pitch, const unsigned char *pV, int v_pitch, int width, int height);
	void readRemoteBuffer(unsigned char *dst_buffer_y, int &y_pitch, unsigned char *dst_buffer_u, int &u_pitch, unsigned char *dst_buffer_v, int &v_pitch, int &width, int &height);

	unsigned char *remote_buffer_y_;
	unsigned char *remote_buffer_u_;
	unsigned char *remote_buffer_v_;
	int remote_y_pitch_;
	int remote_u_pitch_;
	int remote_v_pitch_;

	int remote_width_;
	int remote_height_;

	std::mutex	remote_mutex_;

private:
	int		win_hwnd_;
	int		push_handle_;
	int		pull_handle_;
};