#include "video_player.h"

#include <Windows.h>

#include "SDL.h"

CVideoPlayer::CVideoPlayer(int hWnd, int width, int height)
{
	wnd_handle_ = hWnd;
	video_width_ = width;
	video_height_ = height;

	sdl_window_ = NULL;
	sdl_render_ = NULL;
	sdl_texture_ = NULL;
	sdl_init_ok_ = false;

	video_frame_buffer_y_ = new unsigned char[width*height];
	video_frame_buffer_u_ = new unsigned char[width*height];
	video_frame_buffer_v_ = new unsigned char[width*height];

	frame_y_pitch_ = 0;
	frame_u_pitch_ = 0;
	frame_v_pitch_ = 0;
}
CVideoPlayer::~CVideoPlayer()
{
	delete video_frame_buffer_y_;
	delete video_frame_buffer_u_;
	delete video_frame_buffer_v_;

	video_frame_buffer_y_ = nullptr;
	video_frame_buffer_u_ = nullptr;
	video_frame_buffer_v_ = nullptr;
}

void CVideoPlayer::PlayVideo(int width, int height)
{
	if (!sdl_init_ok_)
	{
		if (!InitSDL(wnd_handle_, width, height))
		{
			return;
		}
	}
	else
	{
		if (width != video_width_ || height != video_height_)
		{
			UnInitSDL();

			if (!InitSDL(wnd_handle_, width, height))
			{
				sdl_init_ok_ = false;
				return;
			}
		}
	}

	PlayVideoWithSDL(video_frame_buffer_y_, frame_y_pitch_, video_frame_buffer_u_, frame_u_pitch_, video_frame_buffer_v_, frame_v_pitch_);

	video_width_ = width;
	video_height_ = height;
}

bool CVideoPlayer::InitSDL(int hWnd, int width, int height)
{
	if (SDL_Init(SDL_INIT_VIDEO) != 0)
	{
		return false;
	}

	sdl_window_ = SDL_CreateWindowFrom((void *)hWnd);
	if (!sdl_window_)
	{
		goto err;
	}

	sdl_render_ = SDL_CreateRenderer(sdl_window_, -1, 0);
	if (!sdl_render_)
	{
		goto err;
	}

	sdl_texture_ = SDL_CreateTexture(sdl_render_, SDL_PIXELFORMAT_IYUV, SDL_TEXTUREACCESS_STREAMING, width, height);
	if (!sdl_texture_)
	{
		goto err;
	}

	sdl_init_ok_ = true;

	video_width_ = width;
	video_height_ = height;

	return true;

err:
	return false;
}
void CVideoPlayer::UnInitSDL()
{
	if (sdl_render_)
	{
		SDL_DestroyRenderer(sdl_render_);
		sdl_render_ = NULL;
	}
	if (sdl_texture_)
	{
		SDL_DestroyTexture(sdl_texture_);
		sdl_texture_ = NULL;
	}
	if (sdl_window_)
	{
		SDL_DestroyWindow(sdl_window_);
		sdl_window_ = NULL;

		//https://www.cnblogs.com/lihaiping/p/4324315.html
		ShowWindow((HWND)wnd_handle_, SW_SHOWNORMAL);
	}

	sdl_init_ok_ = false;
}
void CVideoPlayer::PlayVideoWithSDL(const unsigned char *pY, int yPitch, const unsigned char *pU, int uPitch, const unsigned char *pV, int vPitch)
{
	int nRet = SDL_UpdateYUVTexture(sdl_texture_, NULL, pY, yPitch, pU, uPitch, pV, vPitch);
	if (nRet != 0)
	{
		return;
	}

	int nWidth = 0;
	int nHeight = 0;
	SDL_GetWindowSize(sdl_window_, &nWidth, &nHeight);

	SDL_Rect rc = { 0, 0, nWidth, nHeight };

	SDL_RenderClear(sdl_render_);
	SDL_RenderCopy(sdl_render_, sdl_texture_, NULL, NULL);
	SDL_RenderPresent(sdl_render_);
}