#ifndef __LIB_CONN_MIC_H__
#define __LIB_CONN_MIC_H__

#include "ztypes.h"

namespace offcn
{
	/**
	 *	Init global param
	 */
	ZG_C_API void Initialize(GlobalObserver *observer);

	/**
	 *	UnInit global param
	 */
	ZG_C_API void UnInitialize();

	/**
	 *	Set global param
						"{
							"mic_device_name":"my mic name(utf8)",
							"speaker_device_name":"my speaker name(utf8)",
						}"
	 */
	ZG_C_API void SetParam(const char *param);

	/**
	 *	Create handle
	 *
	 *	@param	param	publish and subscribe param in json format
						"{
							"video_device_name":"my device name(utf8)",
							"video_send_bitrate":300000
						}"
	 *
	 *	@return	role handle id
	 */
	ZG_C_API int CreateRoleHandle(const char *param, ChannelObserver *observer, ZRoleType type);
	/**
	 *	Destroy handle
	 *
	 *	@param	handle		role's handle id
	 *	@param	type		role's type
	 */
	ZG_C_API void DestroyRoleHandle(int handle, ZRoleType type);

	/**
	 *	Input yuv data
	 */
	ZG_C_API void InputVideoData(int handle, const unsigned char *data, int width, int height, int frame_rate);


	/**
	 *	Start/Stop publish stream
	 *
	 *	@param	handle		role handle id
	 */
	ZG_C_API void StartPublish(int handle);
	ZG_C_API void StopPublish(int handle);

	/**
	 *	Start/Stop subscribe stream
	 *
	 *	@param	handle		role handle id
	 */
	ZG_C_API void StartSubscribe(int handle);
	ZG_C_API void StopSubscribe(int handle);
}

#endif