#include "client_manager.h"
#include "msg_data_customize.h"
#include "direct_base_client.h"
#include "direct_push_client.h"
#include "direct_pull_client.h"
#include "offcn_call.h"

static int g_handle = 0;

ClientManager::ClientManager(GlobalObserver *observer)
	:observer_(observer)
{
    net_data_receiver_ = NULL;

    signal_thread_.SetName("client manager signla thread", NULL);
    signal_thread_.Start();
}
ClientManager::~ClientManager()
{

}

/**
 * NetDataReceiveObserver
 */
void ClientManager::NetDataReceiveGetData(SOCKET socket, int handle_id, std::string ip, int port, const char *buffer, int size)
{
    ZMsgDataNetData *data = new ZMsgDataNetData();
    memcpy(data->buffer, buffer, size);
    data->size = size;
    data->socket = socket;
    data->handle = handle_id;

    signal_thread_.Post(RTC_FROM_HERE, this, Client_Msg_Receive_Data, data);
}

void ClientManager::Init()
{
    signal_thread_.Post(RTC_FROM_HERE, this, Client_Msg_Init);
}

int ClientManager::CreatePushHandle(std::string param, ChannelObserver *observer)
{
    ZMsgDataHandle *data = new ZMsgDataHandle();
    data->string_data_  = param;
    data->int_data_     = ++g_handle;
	data->observer_		= observer;

    signal_thread_.Post(RTC_FROM_HERE, this, Client_Msg_Create_Push_Handle, data);

	return data->int_data_;
}
void ClientManager::DestroyPushHandle(int handle)
{
    ZMessageDataInt *data = new ZMessageDataInt();
    data->int_data_ = handle;

    signal_thread_.Post(RTC_FROM_HERE, this, Client_Msg_Destroy_Push_Handle, data);
}
void ClientManager::StartPush(int handle)
{
    ZMessageDataInt *data = new ZMessageDataInt();
    data->int_data_ = handle;

    signal_thread_.Post(RTC_FROM_HERE, this, Client_Msg_Start_Push, data);
}
void ClientManager::StopPush(int handle)
{
    ZMessageDataInt *data = new ZMessageDataInt();
    data->int_data_ = handle;

    signal_thread_.Post(RTC_FROM_HERE, this, Client_Msg_Stop_Push, data);
}
int ClientManager::CreatePullHandle(std::string param, ChannelObserver *observer)
{
    ZMsgDataHandle *data = new ZMsgDataHandle();
    data->string_data_ = param;
    data->int_data_ = ++g_handle;
	data->observer_ = observer;

    signal_thread_.Post(RTC_FROM_HERE, this, Client_Msg_Create_Pull_Handle, data);

	return data->int_data_;
}
void ClientManager::DestroyPullHandle(int handle)
{
    ZMessageDataInt *data = new ZMessageDataInt();
    data->int_data_ = handle;

    signal_thread_.Post(RTC_FROM_HERE, this, Client_Msg_Destroy_Pull_Handle, data);
}
void ClientManager::StartPull(int handle)
{
    ZMessageDataInt *data = new ZMessageDataInt();
    data->int_data_ = handle;

    signal_thread_.Post(RTC_FROM_HERE, this, Client_Msg_Start_Pull, data);
}
void ClientManager::StopPull(int handle)
{
    ZMessageDataInt *data = new ZMessageDataInt();
    data->int_data_ = handle;

    signal_thread_.Post(RTC_FROM_HERE, this, Client_Msg_Stop_Pull);
}

void ClientManager::OnMessage(rtc::Message* msg)
{
    switch(msg->message_id)
    {
        case Client_Msg_Receive_Data:
        {
            ZMsgDataNetData *data = dynamic_cast<ZMsgDataNetData *>(msg->pdata);
            if(data)
            {
                AllClientType::const_iterator itor;
                itor = all_role_.find(data->handle);
                DirectBaseClient *client = dynamic_cast<DirectBaseClient *>(itor->second);
                if(client)
                {
                    client->InputRtpRtcpData(data->socket, data->buffer, data->size);
                }
            }
        }
        break;

        case Client_Msg_Init:
        {
            DoInit();
        }
        break;

        case Client_Msg_Create_Push_Handle:
        {
            ZMsgDataHandle *data = dynamic_cast<ZMsgDataHandle *>(msg->pdata);
            DoCreatePushHandle(data->int_data_, data->string_data_, data->observer_);
        }
        break;

        case Client_Msg_Destroy_Push_Handle:
        {
            ZMessageDataInt *data = dynamic_cast<ZMessageDataInt *>(msg->pdata);
            DoDestroyPushHandle(data->int_data_);
        }
        break;

        case Client_Msg_Start_Push:
        {
            ZMessageDataInt *data = dynamic_cast<ZMessageDataInt *>(msg->pdata);
            DoStartPush(data->int_data_);
        }
        break;
        case Client_Msg_Stop_Push:
        {
            ZMessageDataInt *data = dynamic_cast<ZMessageDataInt *>(msg->pdata);
            DoStopPush(data->int_data_);
        }
        break;

        case Client_Msg_Create_Pull_Handle:
        {
            ZMsgDataHandle *data = dynamic_cast<ZMsgDataHandle *>(msg->pdata);
            DoCreatePullHandle(data->int_data_, data->string_data_, data->observer_);
        }
        break;
        case Client_Msg_Destroy_Pull_Handle:
        {
            ZMessageDataInt *data = dynamic_cast<ZMessageDataInt *>(msg->pdata);
            DoDestroyPullHandle(data->int_data_);
        }
        break;

        case Client_Msg_Start_Pull:
        {
            ZMessageDataInt *data = dynamic_cast<ZMessageDataInt *>(msg->pdata);
            DoStartPull(data->int_data_);
        }
        break;
        case Client_Msg_Stop_Pull:
        {
            ZMessageDataInt *data = dynamic_cast<ZMessageDataInt *>(msg->pdata);
            DoStopPull(data->int_data_);
        }
        break;
    }

    delete msg->pdata;
    msg->pdata = NULL;
}

void ClientManager::DoInit()
{
    offcn_call_ = new OffcnCall();

	offcn_call_->CreateCall(NULL, NULL);

    net_data_receiver_ = new NetDataReceive(this);
    net_data_receiver_->Start();
}

void ClientManager::DoCreatePushHandle(int handle, std::string param, ChannelObserver *observer)
{
    DirectBaseClient *client = new DirectPushClient(offcn_call_, handle, param, true, observer);
    SOCKET socket_audio = client->GetAudioSocket();
    SOCKET socket_video = client->GetVideoSocket();

	net_data_receiver_->AddConnection(socket_audio, handle);
	net_data_receiver_->AddConnection(socket_video, handle);

    all_role_.insert(std::pair<int, DirectBaseClient *>(handle, client));
}
void ClientManager::DoDestroyPushHandle(int handle)
{
    AllClientType::const_iterator itor;

    itor = all_role_.find(handle);
    if(itor != all_role_.end())
    {
        DirectBaseClient *client = (DirectBaseClient *)itor->second;
		if (client)
		{
			SOCKET socket_audio = client->GetAudioSocket();
			SOCKET socket_video = client->GetVideoSocket();

			net_data_receiver_->RemoveConnection(socket_audio);
			net_data_receiver_->RemoveConnection(socket_video);

			delete client;
			all_role_.erase(itor);
		}
    }
}
void ClientManager::DoStartPush(int handle)
{
    AllClientType::const_iterator itor;

    itor = all_role_.find(handle);
    if(itor != all_role_.end())
    {
        DirectPushClient *client = dynamic_cast<DirectPushClient *>(itor->second);
        if(client)
        {
            client->StartPush();
        }
    }
}
void ClientManager::DoStopPush(int handle)
{
	AllClientType::const_iterator itor;

	itor = all_role_.find(handle);
	if (itor != all_role_.end())
	{
		DirectPushClient *client = dynamic_cast<DirectPushClient *>(itor->second);
		if (client)
		{
			client->StopPush();
		}
	}
}

void ClientManager::DoCreatePullHandle(int handle, std::string param, ChannelObserver *observer)
{
	DirectBaseClient *client = new DirectPullClient(offcn_call_, handle, param, false, observer);
	SOCKET socket_audio = client->GetAudioSocket();
	SOCKET socket_video = client->GetVideoSocket();

	net_data_receiver_->AddConnection(socket_audio, handle);
	net_data_receiver_->AddConnection(socket_video, handle);

	all_role_.insert(std::pair<int , DirectBaseClient *>(handle, client));
}
void ClientManager::DoDestroyPullHandle(int handle)
{
	AllClientType::const_iterator itor;

	itor = all_role_.find(handle);
	if (itor != all_role_.end())
	{
		DirectPullClient *client = dynamic_cast<DirectPullClient *>(itor->second);
		if (client)
		{
			SOCKET socket_audio = client->GetAudioSocket();
			SOCKET socket_video = client->GetVideoSocket();

			net_data_receiver_->RemoveConnection(socket_audio);
			net_data_receiver_->RemoveConnection(socket_video);

			delete client;
			all_role_.erase(itor);
		}
	}
}
void ClientManager::DoStartPull(int handle)
{
	AllClientType::const_iterator itor;

	itor = all_role_.find(handle);
	if (itor != all_role_.end())
	{
		DirectPullClient *client = dynamic_cast<DirectPullClient *>(itor->second);
		if (client)
		{
			client->StartPull();
		}
	}
}
void ClientManager::DoStopPull(int handle)
{
	AllClientType::const_iterator itor;

	itor = all_role_.find(handle);
	if (itor != all_role_.end())
	{
		DirectPullClient *client = dynamic_cast<DirectPullClient *>(itor->second);
		if (client)
		{
			client->StopPull();
		}
	}
}