#pragma once

#include "net_data_receive.h"
#include "rtc_base/thread.h"
#include "rtc_base/messagehandler.h"
#include "ztypes.h"

#include <map>

class OffcnCall;
class DirectBaseClient;

class ClientManager : public rtc::MessageHandler, public NetDataReceiveObserver 
{
public:
    ClientManager(GlobalObserver *observer);
    ~ClientManager();

    void Init();

    int CreatePushHandle(std::string param, ChannelObserver *observer);
    void DestroyPushHandle(int handle);
    void StartPush(int handle);
    void StopPush(int handle);

    int CreatePullHandle(std::string param, ChannelObserver *observer);
    void DestroyPullHandle(int handle);
    void StartPull(int handle);
    void StopPull(int handle);

public:
    /**
     * NetDataReceiveObserver
     */
    virtual void NetDataReceiveGetData(SOCKET socket, int handle_id, std::string ip, int port, const char *buffer, int size);

    virtual void OnMessage(rtc::Message* msg);

private:
    enum
    {
        Client_Msg_Init,

        Client_Msg_Create_Push_Handle,
        Client_Msg_Destroy_Push_Handle,

        Client_Msg_Start_Push,
        Client_Msg_Stop_Push,

        Client_Msg_Create_Pull_Handle,
        Client_Msg_Destroy_Pull_Handle,

        Client_Msg_Start_Pull,
        Client_Msg_Stop_Pull,

        Client_Msg_Receive_Data,
    };

    void DoInit();

    void DoCreatePushHandle(int handle, std::string param, ChannelObserver *observer);
    void DoDestroyPushHandle(int handle);
    void DoStartPush(int handle);
    void DoStopPush(int handle);

    void DoCreatePullHandle(int handle, std::string param, ChannelObserver *observer);
    void DoDestroyPullHandle(int handle);
    void DoStartPull(int handle);
    void DoStopPull(int handle);

private:
    rtc::Thread     signal_thread_;

private:
    NetDataReceive  *net_data_receiver_;

private:
    typedef std::map<int, DirectBaseClient *> AllClientType;

    AllClientType                     all_role_;
    OffcnCall                         *offcn_call_;

private:
	GlobalObserver	*observer_;
};