#include "direct_base_client.h"
#include "json/json.h"

DirectBaseClient::DirectBaseClient(int handle, std::string param, bool is_pusher, ChannelObserver *observer)
:handle_(handle),
client_param_(param),
is_pusher_(is_pusher),
observer_(observer)
{
	local_address_ = "";
	local_port_video_ = 0;
	local_port_audio_ = 0;

	remote_address_ = "";
	remote_port_video_ = 0;
	remote_port_audio_ = 0;

    WSADATA wsadata;
	if (WSAStartup(MAKEWORD(2, 2), &wsadata) != 0)
	{
		return;
	}

	ParseParam();

	video_render_ = new CVideoRenderEx(nullptr, this);
}
DirectBaseClient::~DirectBaseClient()
{
    WSACleanup();

	if (video_render_)
	{
		delete video_render_;
		video_render_ = NULL;
	}
}

void DirectBaseClient::ParseParam()
{
	Json::Value root;
	Json::CharReaderBuilder readerBuilder;
	std::unique_ptr<Json::CharReader> const reader(readerBuilder.newCharReader());

	std::string client_param = client_param_;

	const char *start_pos = client_param.c_str();

	std::string err;
	if (!reader->parse(start_pos, start_pos + client_param.length(), &root, &err))
	{
		return;
	}

	if (root["local_address"])
	{
		local_address_ = root["local_address"].asString();
	}

	if (root["local_port_video"])
	{
		local_port_video_ = root["local_port_video"].asInt();
	}
	else
	{
		local_port_video_ = 0;
	}

	if (root["local_port_audio"])
	{
		local_port_audio_ = root["local_port_audio"].asInt();
	}
	else
	{
		local_port_audio_ = 0;
	}

	if (root["remote_address"])
	{
		remote_address_ = root["remote_address"].asString();
	}

	if (root["remote_port_video"])
	{
		remote_port_video_ = root["remote_port_video"].asInt();
	}
	else
	{
		remote_port_video_ = 0;
	}

	if (root["remote_port_audio"])
	{
		remote_port_audio_ = root["remote_port_audio"].asInt();
	}
	else
	{
		remote_port_audio_ = 0;
	}
}
void DirectBaseClient::OnVideoFrame(const unsigned char *pY, int yPitch, const unsigned char *pU, int uPitch, const unsigned char *pV, int vPitch, int width, int height)
{
	if (observer_)
	{
		observer_->OnChannelVideoFrame(handle_, pY, yPitch, pU, uPitch, pV, vPitch, width, height);
	}
}