#include "direct_pull_client.h"
#include "offcn_call.h"
#include "json/json.h"

DirectPullClient::DirectPullClient(OffcnCall *offcn_call, int handle, std::string param, bool is_pusher, ChannelObserver *observer)
:DirectBaseClient(handle, param, is_pusher, observer), offcn_call_(offcn_call)
{
	transport_audio_ = nullptr;
	transport_video_ = nullptr;

	audio_receive_stream_ = nullptr;
	video_receive_stream_ = nullptr;

    socket_audio_ = socket(AF_INET, SOCK_DGRAM, 0);
	socket_video_ = socket(AF_INET, SOCK_DGRAM, 0);
}
DirectPullClient::~DirectPullClient()
{
}

void DirectPullClient::InputRtpRtcpData(SOCKET socket, char *buffer, int size)
{
    if (offcn_call_)
    {
        rtc::CopyOnWriteBuffer packet(buffer, size);
		if (socket == socket_audio_)
		{
			offcn_call_->GetCall()->Receiver()->DeliverPacket(webrtc::MediaType::AUDIO, packet, rtc::TimeMicros());
		}
		else
		{
			offcn_call_->GetCall()->Receiver()->DeliverPacket(webrtc::MediaType::VIDEO, packet, rtc::TimeMicros());
		}
    }
}
bool DirectPullClient::StartPull()
{
	struct sockaddr_in local_addr;
	memset(&local_addr, 0, sizeof(local_addr));

	local_addr.sin_family = AF_INET;
	local_addr.sin_port = htons(local_port_audio_);
	local_addr.sin_addr.s_addr = inet_addr(local_address_.c_str());
	
	int nRet = bind(socket_audio_, (const sockaddr *)&local_addr, sizeof(local_addr));
	if (nRet != 0)
	{
		goto Error;
	}

	local_addr.sin_port = htons(local_port_video_);
	nRet = bind(socket_video_, (const sockaddr *)&local_addr, sizeof(local_addr));
	if (nRet != 0)
	{
		goto Error;
	}

    transport_audio_ = new RtpRtcpTransport(this);
    if(transport_audio_ == nullptr)
    {
        goto Error;
    }

	transport_video_ = new RtpRtcpTransport(this);
	if (transport_video_ == nullptr)
	{
		goto Error;
	}

    audio_receive_stream_ = offcn_call_->CreateAudioReceiveStream(transport_audio_);
    if(audio_receive_stream_ == nullptr)
    {
        goto Error;
    }

	video_receive_stream_ = offcn_call_->CreateVideoReceiveStream(video_render_, transport_video_);
	if (video_receive_stream_ == nullptr)
	{
		goto Error;
	}

    offcn_call_->GetCall()->SignalChannelNetworkState(webrtc::MediaType::AUDIO, webrtc::kNetworkUp);
	offcn_call_->GetCall()->SignalChannelNetworkState(webrtc::MediaType::VIDEO, webrtc::kNetworkUp);

    audio_receive_stream_->Start();
	video_receive_stream_->Start();
    
    return true;

Error:
    StopPull();
    return false;
}
bool DirectPullClient::StopPull()
{
    if (audio_receive_stream_)
    {
        audio_receive_stream_->Stop();

		offcn_call_->GetCall()->DestroyAudioReceiveStream(audio_receive_stream_);
		audio_receive_stream_ = nullptr;
    }

	if (video_receive_stream_)
	{
		video_receive_stream_->Stop();

		offcn_call_->GetCall()->DestroyVideoReceiveStream(video_receive_stream_);
		video_receive_stream_ = nullptr;
	}

    if(transport_audio_)
    {
        delete transport_audio_;
        transport_audio_ = nullptr;
    }

	if (transport_video_)
	{
		delete transport_video_;
		transport_video_ = nullptr;
	}

    if(socket_audio_ > 0)
    {
        closesocket(socket_audio_);
        socket_audio_ = 0;
    }

	if (socket_video_ > 0)
	{
		closesocket(socket_video_);
		socket_video_ = 0;
	}
    
    return true;
}

bool DirectPullClient::SendRtpRtcpData(RtpRtcpTransport *transport, const uint8_t *data, size_t length, bool is_rtcp)
{
	if (remote_address_.size() <= 0)
	{
		return true;
	}

	struct sockaddr_in remote_addr;
	memset(&remote_addr, 0, sizeof(remote_addr));

	remote_addr.sin_family = AF_INET;
	remote_addr.sin_addr.s_addr = inet_addr(remote_address_.c_str());

	int addrLen = sizeof(struct sockaddr_in);

	int nRet = 0;
	if (transport == transport_audio_)
	{
		if (remote_port_audio_ > 0)
		{
			remote_addr.sin_port = htons(remote_port_audio_);
			nRet = sendto(socket_audio_, (const char *)data, length, 0, (struct sockaddr *)&remote_addr, addrLen);
		}
	}
	else
	{
		if (remote_port_video_ > 0)
		{
			remote_addr.sin_port = htons(remote_port_video_);
			nRet = sendto(socket_video_, (const char *)data, length, 0, (struct sockaddr *)&remote_addr, addrLen);
		}
	}

	if (nRet < 0)
	{
		DWORD dw = GetLastError();
		int i = 0;
		return false;
	}

    return true;
}