#include "direct_push_client.h"
#include "api/video_codecs/builtin_video_encoder_factory.h"
#include "offcn_call.h"
#include "encoder_setting.h"
#include "json/json.h"

using cricket::VideoFormat;
const VideoFormat formats[] = { {640, 480, VideoFormat::FpsToInterval(15), cricket::FOURCC_I420} };

DirectPushClient::DirectPushClient(OffcnCall *offcn_call, int handle, std::string param, bool is_pusher, ChannelObserver *observer)
:DirectBaseClient(handle, param, is_pusher, observer), offcn_call_(offcn_call)
{
	transport_video_ = nullptr;
	transport_audio_ = nullptr;

    //create socket
    socket_video_ = socket(AF_INET, SOCK_DGRAM, 0);
    socket_audio_ = socket(AF_INET, SOCK_DGRAM, 0);

    video_encoder_factory_ = webrtc::CreateBuiltinVideoEncoderFactory();
}
DirectPushClient::~DirectPushClient()
{
    StopPush();

}

void DirectPushClient::InputRtpRtcpData(SOCKET socket, char *buffer, int size)
{
    if (offcn_call_)
    {
        rtc::CopyOnWriteBuffer packet(buffer, size);
        if (socket == socket_audio_)
        {
            offcn_call_->GetCall()->Receiver()->DeliverPacket(webrtc::MediaType::AUDIO, packet, rtc::TimeMicros());
        }
        else if(socket == socket_video_)
        {
            offcn_call_->GetCall()->Receiver()->DeliverPacket(webrtc::MediaType::VIDEO, packet, rtc::TimeMicros());
        }
    }
}
bool DirectPushClient::StartPush()
{
	rtc::VideoSinkWants wants;

	struct sockaddr_in local_addr;
	memset(&local_addr, 0, sizeof(local_addr));

	local_addr.sin_family = AF_INET;
	local_addr.sin_port = htons(1237);
	local_addr.sin_addr.s_addr = inet_addr(local_address_.c_str());

	int nRet = bind(socket_audio_, (const sockaddr *)&local_addr, sizeof(local_addr));

	local_addr.sin_port = htons(1236);
	nRet = bind(socket_video_, (const sockaddr *)&local_addr, sizeof(local_addr));

    //create video capture
    video_capturer_ = offcn::OpenVideoCaptureDevice();
    if(video_capturer_ == nullptr)
    {
        return false;
    }

    //create transport
    transport_video_ = new RtpRtcpTransport(this);
    transport_audio_ = new RtpRtcpTransport(this);

    //create video send stream
    video_send_stream_ = offcn_call_->CreateVideoSendStream(video_encoder_factory_.get(), transport_video_, "offcn_stream_send_video");

    //create audio send stream
    audio_send_stream_ = offcn_call_->CreateAudioSendStream(transport_audio_, "offcn_stream_send_audio");

    if(video_send_stream_ == nullptr || audio_send_stream_ == nullptr)
    {
        goto Error;
    }

    offcn_call_->GetCall()->SignalChannelNetworkState(webrtc::MediaType::VIDEO, webrtc::kNetworkUp);
    offcn_call_->GetCall()->SignalChannelNetworkState(webrtc::MediaType::AUDIO, webrtc::kNetworkUp);

    video_send_stream_->Start();
    audio_send_stream_->Start();

    video_send_stream_->SetSource((rtc::VideoSourceInterface<webrtc::VideoFrame> *)video_capturer_.get(), webrtc::DegradationPreference::MAINTAIN_FRAMERATE);
    video_capturer_->Start(formats[0]);

	video_capturer_->AddOrUpdateSink(video_render_, wants);

    return true;

Error:
    StopPush();
    return false;
}
bool DirectPushClient::StopPush()
{
    if(video_capturer_)
    {
        video_capturer_->Stop();

        video_capturer_.reset();
        
        video_capturer_ = nullptr;
    }

    if (video_send_stream_)
    {
        offcn_call_->GetCall()->SignalChannelNetworkState(webrtc::MediaType::VIDEO, webrtc::kNetworkDown);

        video_send_stream_->Stop();
        offcn_call_->GetCall()->DestroyVideoSendStream(video_send_stream_);

        video_send_stream_ = nullptr;
    }
    
    if(audio_send_stream_)
    {
        offcn_call_->GetCall()->SignalChannelNetworkState(webrtc::MediaType::AUDIO, webrtc::kNetworkDown);

        audio_send_stream_->Stop();
        offcn_call_->GetCall()->DestroyAudioSendStream(audio_send_stream_);

        audio_send_stream_ = nullptr;
    }

    if(transport_video_)
    {
        delete transport_video_;
        transport_video_ = nullptr;
    }

    if(transport_audio_)
    {
        delete transport_audio_;
        transport_audio_ = nullptr;
    }
    
    if(socket_video_)
    {
        closesocket(socket_video_);
        socket_video_ = 0;
    }
    
    if(socket_audio_)
    {
        closesocket(socket_audio_);
        socket_audio_ = 0;
    }

    return true;
}

/**
 *  RtpRtcpTransportObserver
 */
bool DirectPushClient::SendRtpRtcpData(RtpRtcpTransport *transport, const uint8_t *data, size_t length, bool is_rtcp)
{
    int nRet = 0;

	struct sockaddr_in remote_addr;
	memset(&remote_addr, 0, sizeof(remote_addr));

	remote_addr.sin_family = AF_INET;
	remote_addr.sin_addr.s_addr = inet_addr(remote_address_.c_str());

    int addrLen = sizeof(struct sockaddr_in);

	int lastError = 0;

    if (transport == transport_video_)
    {
        remote_addr.sin_port = htons(remote_port_video_);
        nRet = sendto(socket_video_, (const char *)data, length, 0, (struct sockaddr *)&remote_addr, addrLen);
		lastError = WSAGetLastError();
    }
    else
    {
        remote_addr.sin_port = htons(remote_port_audio_);
        nRet = sendto(socket_audio_, (const char *)data, length, 0, (struct sockaddr *)&remote_addr, addrLen);
		lastError = WSAGetLastError();
    }

    return true;
}