#include "net_data_receive.h"

NetDataReceive::NetDataReceive(NetDataReceiveObserver *observer)
:observer_(observer)
{
    all_connections_.clear();
}
NetDataReceive::~NetDataReceive()
{}

bool NetDataReceive::Start()
{
    thread_running_ = true;

    recv_thread_ = std::thread(NetDataReceive::RecvThreadCallBack, this);

    return true;
}
bool NetDataReceive::Stop()
{
    thread_running_ = false;

    if(recv_thread_.joinable())
    {
        recv_thread_.join();
    }
    
    return true;
}
void NetDataReceive::RecvThreadCallBack(void *obj)
{
    NetDataReceive *pThis = (NetDataReceive *)obj;
    pThis->RecvThread();
}
void NetDataReceive::RecvThread()
{
    struct timeval tv = {0, 50 * 1000};

    while(thread_running_)
    {
        fd_set read_fds;
		FD_ZERO(&read_fds);

		if (all_connections_.size() <= 0)
		{
			Sleep(10);
			continue;
		}
        
		ConnectionMapType::const_iterator itor;

        //mutex_connections_.lock();
		{
			std::unique_lock<std::mutex> lck(mutex_connections_);

			for (itor = all_connections_.begin(); itor != all_connections_.end(); itor++)
			{
				FD_SET(itor->first, &read_fds);
			}
		}

        int nRet = select(0, &read_fds, NULL, NULL, &tv);
        if (nRet > 0)
        {
			{
				std::unique_lock<std::mutex> lck(mutex_connections_);

				for (itor = all_connections_.begin(); itor != all_connections_.end(); itor++)
				{
					if (FD_ISSET(itor->first, &read_fds))
					{
						struct sockaddr_in client;
						int addr_len = sizeof(client);

						char szBuffer[1500] = { 0 };

						int len = recvfrom(itor->first, szBuffer, 1500, 0, (SOCKADDR*)&client, &addr_len);
						int nPort = ntohs(client.sin_port);
						char *ip = inet_ntoa(client.sin_addr);
						if (len > 0)
						{
							if (observer_)
							{
								observer_->NetDataReceiveGetData(itor->first, itor->second, std::string(ip), nPort, szBuffer, len);
							}
						}
						else
						{
							DWORD dw = GetLastError();
							int i = 0;
						}
					}
				}
			}
        }
        
        //mutex_connections_.unlock();
    }
}

bool NetDataReceive::AddConnection(SOCKET socket, int handle)
{
	char szBuffer[256] = { 0 };
	sprintf_s(szBuffer, 256, "add connection socket = %d", socket);
	OutputDebugStringA(szBuffer);

    //mutex_connections_.lock();
	std::unique_lock<std::mutex> lck(mutex_connections_);

    all_connections_.insert(std::pair<SOCKET, int>(socket, handle));

    //mutex_connections_.unlock();

    return true;
}
bool NetDataReceive::RemoveConnection(SOCKET socket)
{
    ConnectionMapType::const_iterator itor;

    //mutex_connections_.lock();
	std::unique_lock<std::mutex> lck(mutex_connections_);

    itor = all_connections_.find(socket);

    if(itor != all_connections_.end())
    {
        all_connections_.erase(itor);
    }

    //mutex_connections_.unlock();

    return true;
}