#pragma once

#include "call/call.h"
#include "logging/rtc_event_log/rtc_event_log.h"
#include "call/audio_send_stream.h"
#include "call/video_send_stream.h"
#include "api/video_codecs/video_decoder.h"

class OffcnCall
{
public:
    OffcnCall();
    ~OffcnCall();

    static int GetMicroPhoneIndexByName(rtc::scoped_refptr<webrtc::AudioDeviceModule> audio_device, const char *name);
    static int GetPlayoutIndexByName(rtc::scoped_refptr<webrtc::AudioDeviceModule> audio_device, const char *name);

    /**
     * Create call object
     * 
     * @param   playout_dev_name    audio play device name, utf8
     * @param   recorder_dev_name   audio capture device name, utf8
     * 
     * @return true - ok, false - not ok
     */
    bool CreateCall(const char *playout_dev_name, const char *recorder_dev_name);

    /**
     * Return call object pointer
     */
    webrtc::Call *GetCall(){ return call_.get(); }

    webrtc::AudioSendStream *CreateAudioSendStream(webrtc::Transport *transport, const char *cname);
    webrtc::VideoSendStream *CreateVideoSendStream(webrtc::VideoEncoderFactory *video_encoder_factory, webrtc::Transport *transport, const char *cname);
    webrtc::AudioReceiveStream *CreateAudioReceiveStream(webrtc::Transport *transport);
	webrtc::VideoReceiveStream *CreateVideoReceiveStream(rtc::VideoSinkInterface<webrtc::VideoFrame> *videosink, webrtc::Transport *transport);
    
private:
    rtc::scoped_refptr<webrtc::AudioDeviceModule> CreateAudioModule(const char *playout_dev_name, const char *recorder_dev_name);

private:
    webrtc::RtcEventLogNullImpl event_log_;
    std::unique_ptr<webrtc::Call> call_;

	std::unique_ptr<webrtc::VideoDecoder>	video_decoder_;
};