#include "video_render_ex.h"
#include "api/video/i420_buffer.h"
#include "third_party/libyuv/include/libyuv/convert_argb.h"

CVideoRenderEx::CVideoRenderEx(webrtc::VideoTrackInterface *track, CVideoRenderObserver *observer)
	:render_track_(track),
	observer_(observer)
{
	AddVideoSink();
}
CVideoRenderEx::~CVideoRenderEx()
{
}

void CVideoRenderEx::AddVideoSink()
{
	if (render_track_)
	{
		render_track_->AddOrUpdateSink(this, rtc::VideoSinkWants());
	}
}
void CVideoRenderEx::RemoveVideoSink()
{
	if (render_track_)
	{
		render_track_->RemoveSink(this);
	}
	render_track_ = nullptr;
}

void CVideoRenderEx::OnFrame(const webrtc::VideoFrame& video_frame)
{
	rtc::scoped_refptr<webrtc::I420BufferInterface> buffer(video_frame.video_frame_buffer()->ToI420());
	if (video_frame.rotation() != webrtc::kVideoRotation_0)
	{
		buffer = webrtc::I420Buffer::Rotate(*buffer, video_frame.rotation());
	}

	//Play 'buffer'
	if (observer_)
	{
		int w = buffer->width();
		int h = buffer->height();

		int y = buffer->StrideY();
		int u = buffer->StrideU();
		int v = buffer->StrideV();

		observer_->OnVideoFrame(buffer->DataY(), buffer->StrideY(), buffer->DataU(), buffer->StrideU(), buffer->DataV(), buffer->StrideV(), buffer->width(), buffer->height());
	}
}